'''
  Copyleft ITEG IT-Engineers GmbH 2021, 2024
'''

#import base64
#import configparser
import glob
import ipaddress
#import lxc
import os.path
#import socket
import sys

#from clazzes.datetools import import to_datetime_timestamp_millis, 
#from datetime import datetime

#from iteg.pveutils.helpers import formatFileSize
from iteg.pveutils import helpers
from iteg.pveutils import pulogging
#from iteg.pveutils import networkhelpers

log = pulogging.getLogger()

def getPUConfiguration():
    if PUConfiguration.staticPUConfiguration == None:
        PUConfiguration.staticPUConfiguration = PUConfiguration()
    return PUConfiguration.staticPUConfiguration

class PUConfiguration:
    '''
      holds eventual parsed configuration, i.e. /etc/pveutils/environment, /etc/pveutils/conf/*conf
    '''
    
    # static holder for first instance
    staticPUConfiguration = None

    def __init__(self): 
        self.containersByID = {}
        self.containersByHostname = {}
        self.runningContainers = [];
        self.runningContainerNames = [];
        self.vmsByID = {}
        self.vmsByName = {}
        self.parsePVEUtilsConf()

    def parsePVEUtilsConf(self, restrictContainerToName:str=""):
        # pve environment
        self.pveUtilsConfFileName = "/etc/pveutils/pveutils.conf" 
        conf = helpers.readConf(self.pveUtilsConfFileName)
        log.debug("parsePVEUtilsConf(): conf=\n%s" % conf)
        #self.defaultDnsList = conf.get("CTDNS",          "")
        # containers
        self.containersByID = {}
        self.containersByHostname = {}
        for confFileName in glob.glob("/etc/pve/lxc/*.conf"): # we might have to add more paths, for after clustering
            container = PUContainer(confFileName=confFileName, puConfiguration=self)
            if len(container.hostname) == 0:
                self.checkErrors.append("ERROR: Ignoring nameless container from %s" % confFileName)
                continue
            log.debug("parsePVEUtilsConf(): Found container %s (%s)" % (container.id, container.hostname))
            self.containersByID[container.id] = container
            self.containersByHostname[container.hostname] = container
        for ctName in self.containersByID.keys():
            log.debug("parsePVEUtilsConf(): Have container with key %s" % ctName)
        # VMs
        self.vmsByID = {}
        self.vmsByName = {}
        for confFileName in glob.glob("/etc/pve/qemu-server/*.conf"): # we might have to add more paths, for after clustering
            vm = PUVM(confFileName=confFileName, puConfiguration=self)
            if len(vm.name) == 0:
                self.checkErrors.append("ERROR: Ignoring nameless VM from %s" % confFileName)
                continue
            log.debug("parsePVEUtilsConf(): Found VM %s (%s)" % (vm.id, vm.name))
            self.vmsByID[vm.id] = vm
            self.vmsByName[vm.name] = vm

        # done
        log.debug("parsePVEUtilsConf(): Found %d containers" % len(self.containersByID.keys()))

    def getContainer(self, containerRef:str=""):
        if len(containerRef) == 0:
            log.error("getContainer() called with empty container reference.")
            sys.exit(1)
        puContainer = self.containersByID.get(containerRef, None)
        if puContainer != None:
            log.info("Container reference '%s' is a known container ID, using that container %s (%s)" % 
                    (containerRef, puContainer.id, puContainer.hostname))
            return puContainer
        matchingContainer = None
        for puContainer in self.containersByID.values():
            if containerRef in puContainer.hostname:
                if matchingContainer != None:
                    log.error("Container reference '%s' is not a known container ID, but matches multiple container hostnames, returning None.\nFirst 2 matches were:\n %s %s\n %s %s\n" %
                              (containerRef, matchingContainer.id, matchingContainer.hostname, puContainer.id, puContainer.hostname))
                    return None
                matchingContainer = puContainer
        if matchingContainer == None:
            log.error("Container reference '%s' is not a known container ID nor does it match any container hostname, returning None" %
                      containerRef)
            return None
        log.warning("Container reference '%s' not a known container ID, but does match exactly one container hostname, using that container %s (%s)" % 
                    (containerRef, matchingContainer.id, matchingContainer.hostname))
        return matchingContainer     

    def getVM(self, vmRef:str=""):
        if len(vmRef) == 0:
            log.error("getVM() called with empty VM reference.")
            sys.exit(1)
        puVm = self.vmsByID.get(vmRef, None)
        if puVm != None:
            log.info("VM reference '%s' is a known VM ID, using that container %s (%s)" % 
                    (vmRef, puVm.id, puVm.name))
            return puVm
        matchingVm = None
        for puVm in self.vmsByID.values():
            if vmRef in puVm.name:
                if matchingVm != None:
                    log.error("VM reference '%s' is not a known VM ID, but matches multiple VM names, returning None.\nFirst 2 matches were:\n %s %s\n %s %s\n" %
                              (vmRef, matchingVm.id, matchingVm.name, puVm.id, puVm.name))
                    return None
                matchingVm = puVm
        if matchingVm == None:
            log.error("VM reference '%s' is not a known VM ID nor does it match any VM name, returning None" %
                      vmRef)
            return None
        log.warning("VM reference '%s' not a known VM ID, but does match exactly one VM name, using that container %s (%s)" % 
                    (vmRef, matchingVm.id, matchingVm.name))
        return matchingVm     
            
class PUContainer:
    '''
      holds eventual parsed configuration of one container, i.e. one /etc/pve/lxc/*conf
      we might have to add more paths, for after clustering
    '''

    def __init__(self, confFileName:str = "", puConfiguration:PUConfiguration = None):
        self.confFileName = confFileName
        self.id = confFileName[13:].split('.')[0] # name or ID or CTID, sigh
        #log.debug("Parsed filename '%s' to ID '%s'" % (self.confFileName, self.id))
        self.parseConfFile(puConfiguration=puConfiguration)
        self.knownAsRunning = False
        # runtime resources
        self.cur_pid = 0
        self.cur_cpuUse = 0
        self.cur_memUse = 0
        self.cur_kMemUse = 0
        self.cur_txBytes = 0
        self.cur_rxBytes = 0
        self.cur_blkIoUse = 0

    def parseConfFile(self, puConfiguration:PUConfiguration = None):
        conf = helpers.readConf(self.confFileName)
        self.hostname = conf.get("hostname", "")

    def describeRuntimeResources(self, indent:str="  "):
        desc = ""
        #log.debug("Container ID: %s" % self.id)
        desc += indent + "  PID: %d" % self.pid
        desc += indent + "  CPU use: %d" % self.cpuUse
        desc += indent + "  Memory use: %d" % self.memUse
        desc += indent + "  KMem use: %d" % self.kMemUse
        desc += indent + "  TX Bytes: %d" % self.txBytes
        desc += indent + "  RX Bytes: %d" % self.rxBytes
        desc += indent + "  Total Bytes: %d" % (self.txBytes + self.rxBytes)
        desc += indent + "  BlkIO use: %d" % self.blkIoUse

class PUVM:
    '''
      holds eventual parsed configuration of one VM, i.e. one /etc/pve/qemu-server/*conf
      we might have to add more paths, for after clustering
    '''

    def __init__(self, confFileName:str = "", puConfiguration:PUConfiguration = None):
        self.confFileName = confFileName
        self.id = confFileName[21:].split('.')[0] # name or ID or CTID, sigh
        #log.debug("Parsed filename '%s' to ID '%s'" % (self.confFileName, self.id))
        self.serial0 = None
        self.parseConfFile(puConfiguration=puConfiguration)
        self.knownAsRunning = False

    def parseConfFile(self, puConfiguration:PUConfiguration = None):
        conf = helpers.readConf(self.confFileName)
        self.name = conf.get("name", "")
        self.serial0 = conf.get("serial0", None)
        
