'''
  Copyleft ITEG IT-Engineers GmbH 2024
'''

import getopt
import getpass
#import io
#import lxc
#import os
#import os.path
#import shutil
import subprocess
import sys

from iteg.pveutils import dto
from iteg.pveutils import puhelpers
from iteg.pveutils import pulogging

log = pulogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] vmName string\n" % argv0
    usage += "\n"
    usage += "Sends the specified string to the specified PVE/QM VM using qm sendkey.\n"
    usage += "\n"
    usage += "If string is specified as '-' the actual string is requested interactively,\n"
    usage += "without the input beeing echoed, so it's suitable for passwords.\n"
    usage += "\n"
    usage += pulogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -r        Send 'ret' (for 'return') at the end.\n"
    #usage += "    -l        Use lxcObject.start() instead of lxc-start.\n"
    #usage += "              This might become the default without notice.\n"
    return usage


def run_pu_vm_sendstring(argv0,args):
    
    try:
        opts, posParams = getopt.getopt(args, "dhqrtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    #useLxcObject = False
    finishWithRet = False
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o == "-r":
            finishWithRet = True
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))

    puConfiguration = dto.PUConfiguration()
        
    if len(posParams) != 2:
        log.error("Expecting exactly 2 positional parameters, not %d." %
              len(posParams))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    vmReference = posParams[0]
    stringToSend = posParams[1]
    
    puVm = puConfiguration.getVM(vmReference)
    if puVm == None:
        log.error("Unable to find PVE/LXC VM with ID (or unique name part) %s" %
              vmReference)
        sys.exit(1)

    if puVm.serial0 != None:
        log.warning("VM %s (%s) has serial0 configured to %s, you might want to instead call 'qm terminal %s'" %
              (puVm.id, puVm.name, puVm.serial0, puVm.id))
        #sys.exit(1)
        
    if stringToSend == "-":
        stringToSend = getpass.getpass(prompt="Please enter string to send: ")

    lenToSend = len(stringToSend)
    if finishWithRet:
        log.notice("Trying to feed VM %s (%s) with string of length %d + return" %
                 (puVm.id, puVm.name, lenToSend))
        stringToSend += "\n"
    else:
        log.notice("Trying to feed VM %s (%s) with string of length %d (no return)" %
                 (puVm.id, puVm.name, lenToSend))
      
    #keysToSend = []
    i = 0
    for charToSend in stringToSend:
        # https://en.wikibooks.org/wiki/QEMU/Monitor#sendkey_keys
        keyToSend = None
        if charToSend in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
            keyToSend = "shift-%s" % charToSend.lower()
        elif charToSend == "\n":
            keyToSend = "ret"
        else:
            keyToSend = charToSend
        #log.debug("Mapped character '%s' to key '%s', sending this ..." % (charToSend, keyToSend))
        log.debug("Sending key '%s' ..." % (keyToSend))
        i += 1
        if i > lenToSend:
            log.notice("Sending trailing key (%d of %d) ..." % (i, lenToSend))
        else:
            log.notice("Sending key for character %d of %d ..." % (i, lenToSend))
        #keysToSend.append(keyToSend)
        #continue
        cp = subprocess.run(["/usr/sbin/qm", "sendkey", puVm.id, keyToSend]) # extend(lxcStartExtraOptions))
        if cp.returncode != 0:
            log.error("Failed to qm sendkey %s %s, take a look above" % (
                puVm.id, keyToSend))
            sys.exit(1)
        log.debug("Successfully sent %s" %
              (keyToSend))

    if finishWithRet:
        log.notice("Successfully fed VM %s (%s) with string of length %d + return" %
                   (puVm.id, puVm.name, lenToSend))
    else:
        log.notice("Successfully fed VM %s (%s) with string of length %d (no return)" %
                   (puVm.id, puVm.name, lenToSend))

    sys.exit(0)
