'''
  Copyleft ITEG IT-Engineers GmbH 2021, 2024
'''

#import fileinput
#import json
import os
import re
import subprocess
import sys

from iteg.pveutils import dto
from iteg.pveutils import pulogging


log = pulogging.getLogger()


def detectRunningContainers(puConfiguration:dto.PUConfiguration=None):
    try:
        cp = subprocess.run(["/usr/bin/lxc-ls","--running","-1"],
                        stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True)
    except:
        log.error("Failed to execute lxc-ls")
        return;
    if cp.returncode != 0:
        log.error("Failed to execute lxc-ls, got bad returncode %d" % cp.returncode)
        return;
    puConfiguration.runningContainers = []
    puConfiguration.runningContainerNames = []
    for line in cp.stdout.splitlines():
        containerName = line
        puContainer = puConfiguration.getContainer(containerRef=containerName)
        if puContainer == None:
            log.error("Ignoring unknown container '%s'" % containerName)
            continue
        puContainer.knownAsRunning = True
        puConfiguration.runningContainers.append(puContainer)
        puConfiguration.runningContainerNames.append(puContainer.id)
        log.debug("Found running container with name %s" % containerName)
    log.debug("detectRunningContainers(): Found %d containers" % len(puConfiguration.runningContainers))

    
def detectCurrentContainerResources(puContainer:dto.PUContainer=None):
    if puContainer == None:
        raise Exception("ERROR: detectCurrentContainerResources() called with puContainer None")
    if len(puContainer.id) == 0:
        raise Exception("ERROR: detectCurrentContainerResources() called with namelesspuContainer")
        
    try:
        cp = subprocess.run(["/usr/bin/lxc-info","-H","-n",puContainer.id], 
                        stdout=subprocess.PIPE, stderr=subprocess.STDOUT, universal_newlines=True)
    except:
        log.error("Failed to execute lxc-info for container [%s]" % puContainer.id)
        return;
    if cp.returncode != 0:
        log.error("Failed to execute lxc-info for container [%s], got bad returncode %d" % (puContainer.id, cp.returncode) )
        return;
    
    '''
    # lxc-info -H -n vs
    Name:           vs
    State:          RUNNING
    PID:            9960
    IP:             192.168.1.2
    CPU use:        1065729162959
    Memory use:     110596096
    KMem use:       18595840
    Link:           vethUPF72O
    TX bytes:       2147985
    RX bytes:       180064748
    Total bytes:    182212733
    '''
    puContainer.pid = 0
    puContainer.cpuUse = 0
    puContainer.memUse = 0
    puContainer.kMemUse = 0
    puContainer.txBytes = 0
    puContainer.rxBytes = 0
    puContainer.blkIoUse = 0
    
    lineSplitter = re.compile("^ *([A-Z][A-Za-z ]+): +(.*)$")
    for line in cp.stdout.splitlines():
        m = lineSplitter.match(line)
        if m is None:
            log.debug("Ignoring line [%s]" % line)
            continue
        resName = m.group(1)
        resValue = m.group(2)
        #log.debug("Analysig container [%s]\'s resource [%s] with value [%s]" % (puContainer.id, resName, resValue))
        if resName == "State" and resValue != "RUNNING":
            log.error("Cannot detect resources of container [%s] in non-running state [%s]" % (puContainer.id, resValue) )
            return
        if resName == "PID":
            puContainer.pid = int(resValue)
            continue
        if resName == "CPU use":
            puContainer.cpuUse = int(int(resValue)/1000000000)
            continue
        if resName == "Memory use":
            puContainer.memUse = int(resValue)
            continue
        if resName == "KMem use":
            puContainer.kMemUse = int(resValue)
            continue
        if resName == "TX bytes":
            puContainer.txBytes += int(resValue)
            continue
        if resName == "RX bytes":
            puContainer.rxBytes += int(resValue)
            continue
        if resName == "BlkIO use":
            puContainer.blkIoUse += int(resValue)
            continue
        #log.debug("Ignoring container [%s]\'s resource [%s] with value [%s]" % (puContainer.id, resName, resValue))


def execInContainer(puContainer:dto.PUContainer=None, 
                    cmdParts:list=[], 
                    grepFor:str="",
                    waitMaxSeconds:int=-1, # FIXME find way to implement this
                    teeFileName:str=""):
    if puContainer == None:
        raise Exception("ERROR: execInContainer() called without puContainer")
    if len(cmdParts) == 0:
        raise Exception("ERROR: execInContainer() called without cmdParts")
    rc = 0
    
    log.debug("Performing command in PVE container with\n  lxc-attach -n %s -- %s" % (
        puContainer.id, " ".join(cmdParts)))
    fullCmd = []
    fullCmd.extend(["/usr/bin/lxc-attach", "-n", puContainer.id, "--"])
    fullCmd.extend(cmdParts)
    timeout = None
    if waitMaxSeconds > 0:
        timeout = waitMaxSeconds
    log.debug("Performing %s" % " ".join(fullCmd))

    try:
        if (len(grepFor) + len(teeFileName)) == 0:
            #log.debug("execInContainer(pve-attach): Not collecting output.") 
            cp = subprocess.run(fullCmd, timeout=timeout)
            rc = cp.returncode
        else:
            #log.debug("execInContainer(pve-attach): Collecting output.") 
            cp = subprocess.run(fullCmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, timeout=timeout)
            rc = cp.returncode
    except subprocess.TimeoutExpired:
        log.error("TIMEOUT: PVE/LXC container %s (%s) did not finish '%s' within %d seconds." % (
            puContainer.id, puContainer.hostname, " ".join(cmdParts), waitMaxSeconds))
        if rc == 0:
            rc = 1
    log.debug("Back from\n  lxc-attach -n %s -- %s\nwith rc %d" % (
        puContainer.id, " ".join(cmdParts), rc))
    if ((len(grepFor) + len(teeFileName)) == 0) or cp == None:
        #log.debug("execInContainer(useLxcObject): Done early.") 
        return rc
    teeFile = None
    if len(teeFileName) > 0:
        #log.debug("execInContainer(pve-attach): Teeing to '%s'." % teeFileName) 
        teeFile = open(teeFileName, "w")
    if len(grepFor) > 0:
        #log.debug("execInContainer(pve-attach): Grepping for '%s'." % grepFor) 
        pattern = re.compile(grepFor, re.IGNORECASE)
        for catchedLine in cp.stdout.split("\n"):
            if pattern.search(catchedLine):
                print(catchedLine, file=sys.stdout)
                if teeFile != None:
                    print(catchedLine, file=teeFile)
    else:
        #log.debug("execInContainer(pve-attach): Not grepping but still teeing") 
        for catchedLine in cp.stdout.split("\n"):
            print(catchedLine, file=sys.stdout)
            if teeFile != None:
                print(catchedLine, file=teeFile)
    sys.stdout.flush()
    if teeFile != None:
        teeFile.close()
    
    return rc

